@@ -259,6 +259,12 @@ def get_podcast_icon(ydl_mod, sub, metadata):
         except ydl_mod.utils.YoutubeDLError:
             pass
 
+def flatten_entries(metadata, entries):
+    for entry in metadata.get("entries", []):
+        if entry.get("_type") == "playlist":
+            flatten_entries(entry, entries)
+        else:
+            entries.append(entry)
 
 def download(ydl_mod, sub):
     downloaded = []
@@ -268,7 +274,9 @@ def download(ydl_mod, sub):
     if metadata is None:
         print("No metadata found for %s" % sub["name"])
         return
-    entries = metadata.get("entries", [])
+
+    entries = []
+    flatten_entries(metadata, entries)
 
     # Handle podcast icon
     get_podcast_icon(ydl_mod, sub, metadata)
@@ -414,18 +422,34 @@ def write_sub_nfo(sub):
                 }))
 
 
-def write_xml(sub):
-    mds = [
-        metadata_parse(md_file)
-        for md_file in glob.glob(
-            os.path.join(sub["output_dir"], "%s/*.info.json" % sub["name"])
-        )
-    ]
+def write_xml(config, sub):
+    if sub.get("xml_include_files") == 'all':
+        mds = [
+            {
+                "id": os.path.basename(f),
+                "title": '.'.join(os.path.basename(f).split('.')[:-1]),
+                "filename":  os.path.basename(f),
+                "extension": os.path.basename(f).split('.')[-1],
+                "pub_date": date.fromtimestamp(os.path.getmtime(f)),
+                "timestamp": date.fromtimestamp(os.path.getmtime(f)),
+            }
+            for f in glob.glob(
+                os.path.join(sub["output_dir"], "%s/*.*" % sub["name"])
+            ) if os.path.basename(f).split('.')[-1] not in ["json", "jpg", "webp", "meta", "part", "ytdl"]
+        ]
+    else:
+        mds = [
+            metadata_parse(md_file)
+            for md_file in glob.glob(
+                os.path.join(sub["output_dir"], "%s/*.info.json" % sub["name"])
+            )
+        ]
 
     tmpl_args = {
         "last_update": datetime.datetime.now(),
         "channel_title": sub["name"],
-        "channel_link": sub["url"],
+        "channel_link": sub.get("url", ""),
+        "style_rss_feed": config.get("style_rss_feed", True),
         "items": [
             {
                 "id": md["id"],
@@ -446,8 +470,8 @@ def write_xml(sub):
                         md["thumbnail"]
                     ))
                 ]) if md.get("thumbnail") is not None else None,
-                "description": md["description"],
-                "duration": md["duration"],
+                "description": md.get("description", None),
+                "duration": md.get("duration", None),
             }
             for md in mds if md
         ],
